from flask import Blueprint, flash, render_template, request
from flask_wtf import FlaskForm
from wtforms import RadioField
import sys
sys.path.append('~/eval')
from evalweb.web_checker import WebChecker, Language
from evalweb.examples import examples, convert_examples


bp = Blueprint('eval', __name__, url_prefix='/')

tasks = {'DFA': 'DFA',
         'TOT': 'Totální DFA',
         'MIN': 'Minimální DFA',
         'EFA': 'NFA (s ε-kroky)',
         'NFA': 'NFA bez ε-kroků',
         'GRA': 'Regulární gramatika',
         'REG': 'Regulární výraz'}

types = {'DFA': 'DFA',
         'EFA': 'NFA (s ε-kroky)',
         'GRA': 'Regulární gramatika',
         'REG': 'Regulární výraz'}


class TypeForm(FlaskForm):
    make = RadioField('Typ', choices=list(types.items()), default='DFA', coerce=str)


class TaskForm(FlaskForm):
    make = RadioField('Task', choices=list(tasks.items()), default='DFA', coerce=str)


@bp.route('/compare', methods=('GET', 'POST'))
def compare():
    student_form = TaskForm(prefix='student_form')
    teacher_form = TypeForm(prefix='teacher_form')

    if request.method == 'POST' and 'submit_button' in request.form:
        teacher_type = teacher_form.make.data
        teacher_string = request.form['teacher_string']
        student_type = student_form.make.data
        student_string = request.form['student_string']

        checker = WebChecker(student_string=student_string, task=student_type)
        task_solved = checker.compare(teacher_string=teacher_string, teacher_type=teacher_type)
        if not isinstance(task_solved, str):
            return render_template('parsing_error.html', error=task_solved)

        neq = not checker.ok and checker.eq is not None
        extra_word_ce = checker.eq.left_counterexample if neq else None
        missing_word_ce = checker.eq.right_counterexample if neq else None
        inf = checker.eq.inf if neq else None
        if task_solved == "":
            task_solved = "Odpověď splňuje požadovaný formalismus."

        teacher = Language(string=teacher_string, task=teacher_type)
        student = Language(string=student_string, task=student_type)
        is_task = teacher.gen_is_task(task=student_type)

        return render_template('result_compare.html', compare=True, ok=checker.ok,
                               inf=inf, task_solved=task_solved,
                               extra_word_ce=extra_word_ce, missing_word_ce=missing_word_ce,
                               is_task=is_task, teacher=teacher, student=student)
        #flash(error)

    student_example = ""
    teacher_example = ""
    if request.method == 'POST' and 'example_button' in request.form:
        teacher_type = teacher_form.make.data
        student_type = student_form.make.data
        if (teacher_type, student_type) in examples:
            teacher_example, student_example = examples[(teacher_type, student_type)]
        #return redirect(url_for('eval.compare'))

    return render_template('compare.html', student_form=student_form, teacher_form=teacher_form,
                           student_example=student_example, teacher_example=teacher_example)


@bp.route('/convert', methods=('GET', 'POST'))
def convert():
    student_form = TypeForm(prefix='student_form')
    task_form = ConvertForm(prefix='task_form')

    if request.method == 'POST' and 'submit_button' in request.form:
        student_string = request.form['student_string']
        student_type = student_form.make.data
        task = task_form.make.data

        checker = WebChecker(student_string=student_string, task=task)
        output = checker.convert(student_type=student_type)

        student = Language(string=student_string, task=student_type)
        is_task = student.gen_is_task(task=task)

        return render_template('result_convert.html', compare=False, student_string=student_string,
                               student_type=types[student_type], task=tasks[task], output=output, is_task=is_task)
        #flash(error)

    example = ""
    if request.method == 'POST' and 'example_button' in request.form:
        student_type = student_form.make.data
        if student_type in convert_examples:
            example = convert_examples[student_type]
        #return redirect(url_for('eval.convert'))

    return render_template('convert.html', student_form=student_form, task_form=task_form, example=example)


@bp.route('/userref')
def userref():
    return render_template('userref.html')


@bp.route('/about')
def about():
    return render_template('about.html')